/****************************************************************************
 Module
   CoachProtocolFSM.c

 Revision
   1.0.1

 Description
   This does the main processing for the messages we get and
   is the basic state machine for keeping  track or paired status

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "CoachProtocolFSM.h"
#include "PICUartTxFSM.h"
#include "PICUartRxFSM.h"
#include "CommDefines.h"
#include "PIC_SPI_FSM.h"

/*----------------------------- Module Defines ----------------------------*/


#define JERSEY_NUMBER 13

#define ONE_SEC 1225
#define _5HZ 245
#define _10HZ 123

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
static CoachProtocolState_t WaitPairProcessPacket( void );
static CoachProtocolState_t PairedProcessPacket( void );
static CoachProtocolState_t ProtocolIdleProcessPacket( void );

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static CoachProtocolState_t CurrentState;
static uint8_t teamColor = 0; // start on the red team (0 = red, 1 = green)
static uint8_t PlayerAddr[2]; 


// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitCoachProtocolFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitCoachProtocolFSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitProtocolState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostCoachProtocolFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostCoachProtocolFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTemplateFSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunCoachProtocolFSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

  switch ( CurrentState )
  {
      case InitProtocolState :       // If current state is initial Psedudo State
          if ( ThisEvent.EventType == ES_INIT )// only respond to ES_Init
          {
              // this is where you would put any actions associated with the
              // transition from the initial pseudo-state into the actual
              // initial state
			  ES_Timer_InitTimer(CONTROL_TIMER, _5HZ); // for debugging TX Status
              // now put the machine into the actual initial state
              CurrentState = ProtocolIdle;
           }
           break;

      case ProtocolIdle :       // If current state is WaitPair
          switch ( ThisEvent.EventType )
          {
             /*	// test case to test TX Status (want to see continuous transmission)
				case ES_TIMEOUT : 
	  		  {
				if( ThisEvent.EventParam == CONTROL_TIMER)
				{
				  PORTA |= BIT2HI; // set bit RA2 for debugging

				  TX_clearXBeePacket();
			      TX_appendXBeePacket(0x7E);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x07);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0xFF);
			      TX_appendXBeePacket(0xFF);
			      TX_appendXBeePacket(0x04);
			      TX_appendXBeePacket(0xAA);
			      TX_appendXBeePacket(0xAA);
			      TX_appendXBeePacket(0xA7);
			
				  ThisEvent.EventType = ES_REQUEST_SEND;
				  PostPICUartTxFSM(ThisEvent);
			      //printf("ES_TIMEOUT received from Timer %d in Service %d\r\n", 
			       //       ThisEvent.EventParam, MyPriority);
				  
				  ES_Timer_InitTimer(CONTROL_TIMER, _5HZ);
				  PORTA &= BIT2LO;
				}
			  }    
		      break;
			*/

			  case ES_GOOD_PACKET : //If event is event one
              {
                  // Execute action function for state one : event one
                  //PLAYER_DEBUG("Good Packet!\r\n");
                  if( RX_getLock() )
                  {
                     CurrentState = ProtocolIdleProcessPacket();
                     RX_releaseLock();
                  }
                  
               }
               break;

			  case ES_PAIR_REQUEST : // if event is a Pair Request from the Coach C32...
              {
				  //PORTA |= BIT0HI;
				  // sending a pair request
				  TX_clearXBeePacket();
			      TX_appendXBeePacket(0x7E);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x07);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0xFF);
			      TX_appendXBeePacket(0xFF);
			      TX_appendXBeePacket(0x04);
			      TX_appendXBeePacket(0x03);
				  TX_appendXBeePacket( ThisEvent.EventParam );
			      TX_appendXBeePacket( TX_getChecksum() );

				  // actually send the packet
				  ThisEvent.EventType = ES_REQUEST_SEND;
				  PostPICUartTxFSM(ThisEvent);

				  // start pair timer
				  ES_Timer_InitTimer(PAIR_TIMER, ONE_SEC);
				  CurrentState = WaitPair;
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case WaitPair :       // If current state is WaitPair
          switch ( ThisEvent.EventType )
          {
              case ES_GOOD_PACKET : //If event is event one
              {
                  // Execute action function for state one : event one
                  //PLAYER_DEBUG("Good Packet!\r\n");
                  if( RX_getLock() )
                  {
                     CurrentState = WaitPairProcessPacket();
                     RX_releaseLock();
                  }
                  
               }
               break;

			  case ES_TIMEOUT : //If event is event one
              {
					if( ThisEvent.EventParam == PAIR_TIMER ) // if pair timer times out...
					{
						// revert to the idle state
						CurrentState = ProtocolIdle;  
					}
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;

      case Paired :       // If current state is WaitPair
          switch ( ThisEvent.EventType )
          {
              case ES_GOOD_PACKET : //If event is event one
              {
                  // Execute action function for state one : event one
                  //PLAYER_DEBUG("Good Packet!\r\n");
                  if( RX_getLock() )
                  {
                     CurrentState = PairedProcessPacket();
                     RX_releaseLock();
                  }
                  
               }
               break;

              case ES_RESET : //If event is event one
              {
					// revert to idle state
					CurrentState = ProtocolIdle;
					// clear the current status after losing pair	
					ClearCurrentStatus();
					// explicitly stop the control and heartbeat timers
					ES_Timer_StopTimer(CONTROL_TIMER);
					ES_Timer_StopTimer(HEARTBEAT_TIMER);

					// send reset packet to the player
				  TX_clearXBeePacket();
			      TX_appendXBeePacket(0x7E);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x07);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(PlayerAddr[0]);
			      TX_appendXBeePacket(PlayerAddr[1]);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x04);
			      TX_appendXBeePacket( ThisEvent.EventParam );
			      TX_appendXBeePacket( TX_getChecksum() );

				  // actually send the packet
				  ThisEvent.EventType = ES_REQUEST_SEND;
				  PostPICUartTxFSM(ThisEvent);
               }
               break;

              case ES_TAG_OUT : //If event is event one
              {
					// send tag out packet to the player
				  TX_clearXBeePacket();
			      TX_appendXBeePacket(0x7E);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x07);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(0x01);
			      TX_appendXBeePacket(PlayerAddr[0]);
			      TX_appendXBeePacket(PlayerAddr[1]);
			      TX_appendXBeePacket(0x00);
			      TX_appendXBeePacket(0x02);
			      TX_appendXBeePacket( ThisEvent.EventParam );
			      TX_appendXBeePacket( TX_getChecksum() );

				  // actually send the packet
				  ThisEvent.EventType = ES_REQUEST_SEND;
				  PostPICUartTxFSM(ThisEvent);
               }
               break;


              case ES_TIMEOUT : //If event is event one
              {
				if( ThisEvent.EventParam == HEARTBEAT_TIMER ) // if heartbeat timer expires...
				{
					// revert to idle state
					CurrentState = ProtocolIdle;
					// clear the current status after losing pair	
					ClearCurrentStatus();
					// explicitly stop the control timer
					ES_Timer_StopTimer(CONTROL_TIMER);
				} else if ( ThisEvent.EventParam == CONTROL_TIMER ) // if control timer expires...
				{
					  //PORTA ^= BIT0HI; //debug: check the transmit freq, should be 5Hz
					  // send control packet!
					  if (TX_clearXBeePacket())
					  {
						  TX_appendXBeePacket(0x7E);
					      TX_appendXBeePacket(0x00);
					      TX_appendXBeePacket(0x0B);
					      TX_appendXBeePacket(0x01);
					      TX_appendXBeePacket(0x01);
					      //TX_appendXBeePacket(0x21);
					      //TX_appendXBeePacket(0x8D);
					      TX_appendXBeePacket(PlayerAddr[0]);
					      TX_appendXBeePacket(PlayerAddr[1]);
					      TX_appendXBeePacket(0x00);
					      TX_appendXBeePacket(0x01);
						  TX_appendXBeePacket( GetCurrentSpeed() );
						  TX_appendXBeePacket( GetCurrentOrientation() );
						  TX_appendXBeePacket( GetCurrentAux() );
						  TX_appendXBeePacket( 0x00 ); // whimsy byte
					      TX_appendXBeePacket( 0x00 ); // translation
					      TX_appendXBeePacket( TX_getChecksum() );
	
					  // actually send the packet
					  ThisEvent.EventType = ES_REQUEST_SEND;
					  PostPICUartTxFSM(ThisEvent);
						}
	
					// re-initialize control timer
					ES_Timer_InitTimer(CONTROL_TIMER, _5HZ);
				}
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states

  }                                   // end switch on Current State
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryCoachProtocolFSM

 Parameters
     None

 Returns
     CoachProtocolState_t The current state of the CoachProtocol state machine

 Description
     returns the current state of the CoachProtocol state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
CoachProtocolState_t QueryCoachProtocolFSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

/****************************************************************************
 Function
     ProtocolIdleProcessPacket

 Parameters
     None

 Returns
     None

 Description
     Implment a decision tree to determin the contents of the packet and post appropriate
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
 static CoachProtocolState_t ProtocolIdleProcessPacket( void )
 {
   ES_Event TXStatusEvent;
   //if API identifier is TX status
   if(RX_getXBeePacket(XBEE_API) != 0x89)
   {
		return ProtocolIdle;
   }

   //if TX was a success
   if(RX_getXBeePacket(TXS_STATUS) == 0)
   {
		TXStatusEvent.EventType = ES_TX_STATUS_GOOD;
		PostPICUartTxFSM(TXStatusEvent);
		//PORTC ^= BIT0HI;
		//PLAYER_DEBUG("TX Status Good\r\n");
		return ProtocolIdle;
	}
	else
	{
		TXStatusEvent.EventType = ES_TX_STATUS_ERROR;
		PostPICUartTxFSM(TXStatusEvent);
		//PLAYER_DEBUG("TX Status Error\r\n");
		return ProtocolIdle;
	}
  }

/****************************************************************************
 Function
     WaitPairProcessPacket

 Parameters
     None

 Returns
     None

 Description
     Implment a decision tree to determin the contents of the packet and post appropriate
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
 static CoachProtocolState_t WaitPairProcessPacket( void )
 {
   ES_Event TXStatusEvent;
   //if API identifier is TX status
   if(RX_getXBeePacket(XBEE_API) == 0x89)
   {
      //if TX was a success
      if(RX_getXBeePacket(TXS_STATUS) == 0)
      {
         TXStatusEvent.EventType = ES_TX_STATUS_GOOD;
         PostPICUartTxFSM(TXStatusEvent);
         //PLAYER_DEBUG("TX Status Good\r\n");
         return WaitPair;
      }
      else
      {
         TXStatusEvent.EventType = ES_TX_STATUS_ERROR;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Error\r\n");
         return WaitPair;
      }
   }//end TX Status IF

   //is it a broadcast?
   if((RX_getXBeePacket(RX_OPT) & BIT2HI) == 0x04)
   {
    //PLAYER_DEBUG("Broadcast received\r\n");
      return WaitPair;   
   }
   
   //is it a pair response?
   if(RX_getXBeePacket(RX_DATA) != HDR_PAIR_RESP )
   {
    //PLAYER_DEBUG("Not a pair response\r\n");
      return WaitPair;
   }

	// is it a pair response color error?
	if((RX_getXBeePacket(RX_DATA + 1) & BIT1HI))
	{
		// COLOR ERROR DETECTED
		return WaitPair;
	}

	// is it a pair response pairing error (player already paired)?
	if((RX_getXBeePacket(RX_DATA + 1) & BIT2HI))
	{
		// PAIRING ERROR DETECTED
		return WaitPair;
	}

	// is accept bit set?
	if((RX_getXBeePacket(RX_DATA + 1) & BIT0HI))
	{
		// pairing accepted
		// store player radio address
		PlayerAddr[0] = RX_getXBeePacket(RX_ADDR0);
		PlayerAddr[1] = RX_getXBeePacket(RX_ADDR1);
		// start heartbeat timer
		ES_Timer_InitTimer(HEARTBEAT_TIMER, ONE_SEC);
		// start control timer
		ES_Timer_InitTimer(CONTROL_TIMER, _5HZ);
		return Paired;
	}

   return WaitPair;
 }


/****************************************************************************
 Function
     PairedProcessPacket

 Parameters
     None

 Returns
     None

 Description
     Implement a decision tree to determine the contents of the packet and post appropriate
     states and event
 Notes

 Author
     N. Martelaro, 05/11/14, 21:36
****************************************************************************/
 static CoachProtocolState_t PairedProcessPacket( void )
 {
   ES_Event TXStatusEvent;
   //if API identifier is TX status
   if(RX_getXBeePacket(XBEE_API) == 0x89)
   {
      //if TX was a success
      if(RX_getXBeePacket(TXS_STATUS) == 0)
      {
         TXStatusEvent.EventType = ES_TX_STATUS_GOOD;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Good\r\n");
         return Paired;
      }
      else
      {
         TXStatusEvent.EventType = ES_TX_STATUS_ERROR;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Error\r\n");
         return Paired;
      }
   }//end TX Status IF
   
   //is it a broadcast?
   if((RX_getXBeePacket(RX_OPT) & BIT2HI) == 0x04)
   {
		// don't care about broadcast messages
		return Paired;
   }
   
   //Not a broadcast
   //Is it from our player?
   if(RX_getXBeePacket(RX_ADDR0) != PlayerAddr[0])
   {
    //PLAYER_DEBUG("Wrong Player MSB\r\n");
      return Paired;
   }
   if(RX_getXBeePacket(RX_ADDR1) != PlayerAddr[1])
   {
    //PLAYER_DEBUG("Wrong Player LSB\r\n");
      return Paired;
   }
   
   // is it a status byte?
   if(RX_getXBeePacket(RX_DATA) != HDR_STATUS)
   {
		// not a status header
		return Paired;
   }

	// if it is a status, we reset the heartbeat timer
	ES_Timer_InitTimer(HEARTBEAT_TIMER, ONE_SEC);
    // PORTA ^= BIT0HI; //toggle for rx of status packet
	// we update the status info in PIC_SPI_FSM
	// augmenting status byte with a "pairing" bit (BIT6HI)
    SetCurrentStatus( RX_getXBeePacket(RX_DATA + 1) | BIT6HI );
   return Paired;   
}