/****************************************************************************
 Module
   ES_Port.c

 Revision
   1.0.1

 Description
   This is the sample file to demonstrate adding the hardware specific 
   functions to the Events & Services Framework. This sample is also used
   as the file for the port to the Freescale MC9S12C32 processor.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 08/13/13 12:42 jec     moved the hardware specific aspects of the timer here
 08/06/13 13:17 jec     Began moving the stuff from the V2 framework files
****************************************************************************/
#include "ES_Port.h"
#include "ES_Types.h"
#include "ES_Timers.h"
#include "PICUartTxFSM.h"
#include "PIC_SPI_FSM.h"
#include "EventCheckers.h"

// TickCount is used to track the number of timer ints that have occurred
// since the last check. It should really never be more than 1, but just to
// be sure, we increment it in the interrupt response rather than simply 
// setting a flag. Using this variable and checking approach we remove the
// need to post events from the interrupt response routine. This is necesary
// for compilers like HTC for the midrange PICs which do not produce re-entrant
// code so cannot post directly to the queues from within the interrupt resp.
static uint8_t TickCount;

/****************************************************************************
 Function
     _Timer_HW_Init
 Parameters
     unsigned char Rate set to one of the TMR_RATE_XX values to set the
     Tick rate
 Returns
     None.
 Description
     Initializes the timer module by setting up the RTI with the requested
    rate
 Notes
     modify as required to port to other timer hardware
 Author
     J. Edward Carryer, 02/24/97 14:23
****************************************************************************/
void _HW_Timer_Init(TimerRate_t Rate)
{
   TMR0 = 0X00;						/* clear Timer0 */
   OPTION_REG = 0;    	/* set RTI Rate to 1:16 prescale */
   PS0=1;
   PS1=1;
	//INTCON = (GIE | T0IE);           /* clear any pending int */
   GIE = 1;
   T0IE = 1;									/* Enable global interurupts */
   //CRGINT |= _S12_RTIE;             /* enable the RTI int */
   //EnableInterrupts;                /* make sure that interrupts are enabled*/
}

/****************************************************************************
 Function
     _Timer_INT_Resp
 Parameters
     none
 Returns
     None.
 Description
     interrupt response routine for the tick interupt that will allow the
     framework timers to run.
 Notes
     As currently (8/13/13) implemented this does not actually post events
     but simply sets a flag to indicate that the interrupt has occurred.
     the framework resonse is handled below in _HW_Process_Pending_Ints
 Author
     J. Edward Carryer, 08/13/13 13:29
****************************************************************************/
void interrupt _HW_Timer_INT_Resp(void)
{
// add your timer tick interrupt processing here
	// incrementiong the hardware ES timer
	if(T0IF)
	{
		T0IF = 0;   /* clear the source of the int */
	 	++TickCount;          /* flag that it occured and needs a response */
	}
	// received byte via SPI
	if(SSPIF && SSPIE)
	{
		ReceivedBytes[ByteIndex] = SSPBUF;
		SSPBUF = CurrentStatus;
		//SSPBUF = 0xC7; // load constant into SSPBUF on every interrupt to verify SPI	
		// increment ByteIndex (mod 2)
		ByteIndex = ++ByteIndex % 2;
		// clear SSPIF flag
		SSPIF = 0;
	}
	// transmistting a msg vis UART
	if(TXIF && TXIE)
	{
    	if( TX_index >= TX_length)
         {
            //CurrentState = WaitTXStatus;
            TXIE = 0; //disableTXInterrupt();
			SoftwareTXFlag = 1;
         }
         else
         {
			// TXIF gets cleared by writing to TXREG
            TXREG = XBeePacket[TX_index]; // transmit packet
			TX_index = TX_index + 1;
         }
	}   
}

/****************************************************************************
 Function
     _HW_Process_Pending_Ints
 Parameters
     none
 Returns
     always true.
 Description
     processes any pending interrupts (actually the hardware interrupt already
     occurred and simply set a flag to tell this routine to execute the non-
     hardware response) 
 Notes
     While this routine technically does not need a return value, we always
     return true so that it can be used in the conditional while() loop in
     ES_Run. This way the test for pending interrupts get processed after every
     run function is called and even when there are no queues with events.
     This routine could be expanded to process any onther interrupt sources
     that you would like to use to post events to the framework services.
 Author
     J. Edward Carryer, 08/13/13 13:27
****************************************************************************/
bool _HW_Process_Pending_Ints( void )
{
   while (TickCount > 0)
   {
      /* call the framework tick response to actually run the timers */
      ES_Timer_Tick_Resp();  
      TickCount--;
   }
   return true; // always return true to allow loop test in ES_Run to proceed
}