/****************************************************************************
 Module
   PICUartRxFSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing flat state machines under the
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
//#include <stdio.h>
#include "PICUartRxFSM.h"
#include "CoachProtocolFSM.h"

/*----------------------------- Module Defines ----------------------------*/

//#define UART_RX_DEBUG_FLAG  0
//#define UART_RX_DEBUG if(UART_RX_DEBUG_FLAG) printf

#define XBEE_RXPACKET_MAX_LENGTH 15
#define	SAFETY_INTERVAL			 40

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
// static void ReadByte( void );
bool RX_clearXBeePacket( void );
bool RX_appendXBeePacket( uint8_t newByte );

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match that of enum in header file
static PICUartRxState_t CurrentState;
static uint8_t NewByte; //create a variable for the new byte we recieve
static uint8_t ByteCount; //create a variable for the byte loop counter
static uint8_t CheckSumCount; //create a variable to keep a roling count on the checksum

static uint8_t XBeePacket[XBEE_RXPACKET_MAX_LENGTH]; //create an array to store bytes
static uint8_t length;
static uint8_t index;

static bool    lockAvailable;


// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitPICUartRxFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitPICUartRxFSM ( uint8_t Priority )
{
  ES_Event ThisEvent;
  
  length = 0;
  index = 0;
  lockAvailable = 1;

  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitPRXState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostPICUartRxFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostPICUartRxFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunPICUartRxFSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
   State machine will cycle through recieving a XBee packet.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunPICUartRxFSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
  

  switch ( CurrentState )
  {
      case InitPRXState :       // If current state is initial Psedudo State
          if ( ThisEvent.EventType == ES_INIT )// only respond to ES_Init
          {
              CurrentState = Wait0x7E;
           }
           break;

      case Wait0x7E :       // If current state is state one
          switch ( ThisEvent.EventType )
          {
              case ES_XBEE_RX : //If event is event one
              {
                  // Execute action function for Wait0x7E : RX flag is set
                  NewByte = (uint8_t)ThisEvent.EventParam; // This was read from RCREG in the event checker ReadByte(); // read the UART RX register data
                  //UART_RX_DEBUG("RX: %x\r\n", NewByte); // print the NewByte for debugging
                  
                  if(NewByte == 0x7E)
                  {
                     if(!RX_getLock())                      // If we can't access the buffer, skip
                        CurrentState = Wait0x7E;            
                     else
				     {
						RX_clearXBeePacket();
						if(RX_appendXBeePacket(NewByte))  // If we can add to the buffer, continue
                        	CurrentState = WaitMSB; // move to wait for WaitMSB 
							// start RX_SAFETY_TIMER on state transition
							//ES_Timer_InitTimer(RX_SAFETY_TIMER, SAFETY_INTERVAL);
				     }
                  }
                  else
                  {
                     CurrentState = Wait0x7E; // else ignore and return to idle
                  }
                  //Decide what the next state will be
                  
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case WaitMSB :       // If current state is state one
          switch ( ThisEvent.EventType )
          {
              case ES_XBEE_RX : //If event is event one
              {
                  // Execute action function for Wait0x7E : RX flag is set
                  NewByte = (uint8_t)ThisEvent.EventParam; // This was read from RCREG in the event checker ReadByte(); // read the UART RX register data
                  //UART_RX_DEBUG("RM: %x\r\n", NewByte); // print the NewByte for debugging
                  
                  if(NewByte == 0x00)
                  {
                     if(RX_appendXBeePacket(NewByte))
                        CurrentState = WaitLSB;
						// start RX_SAFETY_TIMER on state transition
						//ES_Timer_InitTimer(RX_SAFETY_TIMER, SAFETY_INTERVAL);
                  }
                  else
                  {
                     CurrentState = Wait0x7E; // else ignore and return to idle
                  }
                  //Decide what the next state will be
                  
               }
               break;
/*
              case ES_TIMEOUT : // if RX_SAFETY_TIMER times out...
              {	
					CurrentState = Wait0x7E;		
              }
              break;
*/

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case WaitLSB :       // If current state is state one
          switch ( ThisEvent.EventType )
          {
              case ES_XBEE_RX : //If event is event one
              {
                  // Execute action function for Wait0x7E : RX flag is set
                  NewByte = (uint8_t)ThisEvent.EventParam; // This was read from RCREG in the event checker ReadByte(); // read the UART RX register data
                  //UART_RX_DEBUG("RL: %x\r\n", NewByte); // print the NewByte for debugging
                  ByteCount = NewByte; // set the value of the NewByte to be a loop counter
                                       // this is how many bytes we will get
                  if(RX_appendXBeePacket(NewByte))
			      {
                     CurrentState = SuckBody; // move to wait for WaitMSB  
					 // start RX_SAFETY_TIMER on state transition
					 //ES_Timer_InitTimer(RX_SAFETY_TIMER, SAFETY_INTERVAL);
			      }
                  else
                     CurrentState = Wait0x7E; // Recover if error                  
               }
               break;
/*
              case ES_TIMEOUT : // if RX_SAFETY_TIMER times out...
              {	
					CurrentState = Wait0x7E;		
              }
              break;
*/

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case SuckBody :       // If current state is state one
          switch ( ThisEvent.EventType )
          {
              case ES_XBEE_RX : //If event is event one
              {
                  // Execute action function for SuckBody : RX flag is set
                  static uint8_t ByteCounter = 0; 
                  NewByte = (uint8_t)ThisEvent.EventParam; // This was read from RCREG in the event checker ReadByte(); // read the UART RX register data
                  //UART_RX_DEBUG("RB: %x\r\n", NewByte); // print the NewByte for debugging
                  RX_appendXBeePacket(NewByte); // save the data into a packet buffer
                  
                  ByteCounter++; // increment the ByteCounter
                  CheckSumCount += NewByte; // accumulate CheckSum
                  
                  if(ByteCounter >= ByteCount)  // if ByteCounter is equal to ByteCount, 
                  {  
                     ByteCounter = 0;    //reset the ByteCounter
                     CurrentState = WaitCheckSum; //move to WaitCheckSum
					 // start RX_SAFETY_TIMER on state transition
					 //ES_Timer_InitTimer(RX_SAFETY_TIMER, SAFETY_INTERVAL);
                  }
                  else
                  {
                     CurrentState = SuckBody; // keep getting data
                  }  // else, stay in Suck Body 
                    
                  //Decide what the next state will be
                  
               }
               break;
/*
              case ES_TIMEOUT : // if RX_SAFETY_TIMER times out...
              {	
					CurrentState = Wait0x7E;		
              }
              break;
*/
              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case WaitCheckSum :       // If current state is state one
          switch ( ThisEvent.EventType )
          {
              case ES_XBEE_RX : //If event is event one
              {
                  // Execute action function for SuckBody : RX flag is set 
                  NewByte = (uint8_t)ThisEvent.EventParam; // This was read from RCREG in the event checker ReadByte(); // read the UART RX register data
                  //UART_RX_DEBUG("RC: %x\r\n", NewByte); // print the NewByte for debugging
                  
                  if(NewByte + CheckSumCount == 0xFF) //if CheckSumCount + CheckSum is eqaul to 0xFF
                  {
                    //announce good packet
					ThisEvent.EventType = ES_GOOD_PACKET;
					PostCoachProtocolFSM(ThisEvent);

                    //UART_RX_DEBUG("Good Packet!\r\n");
					PORTA ^= BIT1HI;  //toggle bit 1 on porta as a good packet heartbeat 
                    CheckSumCount = 0; //reset the CheckSumCount 
                    CurrentState = Wait0x7E; // and go back to Wait0x7E
                  }
                  else
                  {
                     //UART_RX_DEBUG("CHECKSUM ERROR!\r\n");
					 //PORTA ^= BIT2HI;
                     CurrentState = Wait0x7E; //else go back to idle (silent ignore for now -- error checking needed) 
                  }                   
                  //Decide what the next state will be 
               }
               break;
/*
              case ES_TIMEOUT : // if RX_SAFETY_TIMER times out...
              {	
					CurrentState = Wait0x7E;		
              }
              break;
*/

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states

  }                                   // end switch on Current State
  
  // Emulate state entry events without hierarchical organization
  if(CurrentState == Wait0x7E)
  {
      RX_releaseLock();
      index = 0;
	  CheckSumCount =0; 
  }
  
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryPICUartRxSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
PICUartRxState_t QueryPICUartRxFSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
 
 // Function to read the byte from the UART data reguister
 // This allows the code to be more portable to the PIC
 /* static void ReadByte( void )
 {
   NewByte = RCREG; // read the data byte
   return;    // return the NewByte
 }
*/
 
 bool RX_clearXBeePacket( void )
{
   if(CurrentState != Wait0x7E)
   {
      return false;
   }
   else
   {
      length = 0;
      return true;  
   }
}


bool RX_appendXBeePacket( uint8_t newByte )
{
   if ((length == XBEE_RXPACKET_MAX_LENGTH))
   {
      return false;
   }
   else
   {
      XBeePacket[length++] = newByte;
      return true;
   }
}

uint8_t RX_getXBeePacket( uint8_t _index )
{
   #ifdef _DEBUG
   if(_index >= length)
   {
      //printf("ERROR: Index out-of-bounds in E128UartRxFSM.c\n\r");
      return 0;
   }
   #endif
   
   return XBeePacket[_index];
}

uint8_t getLength( void )
{
   return length;
}

bool RX_getLock( void )
{
   if(lockAvailable)
   {
      lockAvailable = false;
      return true;
   }
   else
   {
      #ifdef _DEBUG
      //printf("Could not get lock to access RX Buffer\r\n");
      #endif
      return false;
   }
}

void RX_releaseLock( void )
{
   lockAvailable = true;
}

