/****************************************************************************
 Module
   PICUartTxFSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing flat state machines under the
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include <stdio.h>
#include "PICUartTxFSM.h"

/*----------------------------- Module Defines ----------------------------*/
#define XBEE_TXPACKET_MAX_LENGTH 15

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

static void SendByte( uint8_t NewByte );
static void enableTXInterrupt( void );
static void disableTXInterrupt( void );

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match that of enum in header file
static PICUartTxState_t CurrentState;
//static uint8_t NewByte; //create a variable for the new byte we recieve
static uint8_t ByteCount; //create a variable for the byte loop counter
uint8_t XBeePacket[XBEE_TXPACKET_MAX_LENGTH]; //create an array to send
uint8_t TX_length;
uint8_t TX_index;


// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitPICUartTxFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitPICUartTxFSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  TX_length = 0;
  TX_index = 0;
  
  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitPTXState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostPICUartTxFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostPICUartTxFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunPICUartTxFSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
   State machine will cycle through recieving a XBee packet.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunPICUartTxFSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
  

  switch ( CurrentState )
  {
      case InitPTXState :       // If current state is initial Psedudo State
          if ( ThisEvent.EventType == ES_INIT )// only respond to ES_Init
          {
              // this is where you would put any actions associated with the
              // transition from the initial pseudo-state into the actual
              // initial state

              // now put the machine into the actual initial state
              CurrentState = Idle;
           }
           break;
      case Idle :
         if ( ThisEvent.EventType == ES_REQUEST_SEND)
         {
            if(TX_length == 0)
            {
               //UART_TX_DEBUG("ERROR: Request to send with zero length\r\n");
            }
            else
            {
               CurrentState = SendMSG;
			   //PORTC |= BIT1HI;
               enableTXInterrupt();
            }    
         }
         break;
         
      case SendMSG :
         if ( ThisEvent.EventType == ES_TX_DISABLE )
         {
            //PORTC &= BIT1LO;
			CurrentState = Idle; //this was changed to avoid missing sends
			//CurrentState = WaitTXStatus;
         }
         break;
         
      case WaitTXStatus :
      {
         if ( ThisEvent.EventType == ES_TX_STATUS_GOOD)
         {
            CurrentState = Idle;
         }
         else if ( ThisEvent.EventType == ES_TX_STATUS_ERROR)
         {
            CurrentState = Idle;
         }
      }break;

     
  }                                   // end switch on Current State
  
  // Emulate state entry events without hierarchical organization
  if(CurrentState == Idle)
  {
      TX_index = 0;
  }
  
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryPICUartTxSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
PICUartTxState_t QueryPICUartTxFSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
 
 // Function to write the byte to the UART data register
/*
static void SendByte( uint8_t NewByte )
{
   TXREG = NewByte; // write the data byte
   return;    // return
}
*/

static void enableTXInterrupt( void )
{
   //PIE1 |= 1<<TXIE; //enable the TX interrupt
	TXIE = 1;
}
/*
static void disableTXInterrupt( void )
{
   PIE1 &= ~(1<<TXIE);
}
*/
/***************************************************************************
 Getters/Setters
 ***************************************************************************/

bool TX_clearXBeePacket( void )
{
   if(CurrentState != Idle)
   {
      return false;
   }
   else
   {
      TX_length = 0;
      return true;  
   }
}


bool TX_appendXBeePacket( uint8_t newByte )
{
   if ((TX_length == XBEE_TXPACKET_MAX_LENGTH) || (CurrentState != Idle))
   {
      return false;
   }
   else
   {
      XBeePacket[TX_length] = newByte;
	  TX_length = TX_length + 1;
      return true;
   }
}

uint8_t TX_getChecksum( void )
{
   uint8_t chk = 0;
   uint8_t i;
   
   for(i = 3; i < TX_length; i++)
   {
      chk += XBeePacket[i];
   }
   
   return 0xFF - chk;
}

uint8_t TX_getXBeePacket( uint8_t _index )
{
   #ifdef _DEBUG
   if(_index >= TX_length)
   {
      //printf("ERROR: Index out-of-bounds in E128UartTxFSM.c\n\r");
      return 0;
   }
   #endif
   
   return XBeePacket[_index];
}

uint8_t TX_getLength( void )
{
   return TX_length;
}

/***************************************************************************
 interrupts
 ***************************************************************************/

/*void interrupt _Vec_sci1 TX_ISR( void )
{
     if(SCI1SR1 & _S12_TDRE)
     {
         if( index >= length)
         {
            CurrentState = WaitTXStatus;
            disableTXInterrupt();
         }
         else
         {
            SendByte(XBeePacket[index++]);
         }
     }
}
*/