/****************************************************************************
 Module
   PIC_SPI_FSM.c

 Revision
	1.0.0

 Description
	SPI FSM for the COACH PIC16F690.
   
 Notes
 
 ME 218c -- Team 13: Enchantment Under the Sea
 Author: Buddy Gardineer 
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC_SPI_FSM.h"
#include "SPI.h"
#include "CoachProtocolFSM.h"

/*----------------------------- Module Defines ----------------------------*/

// header byte definitions
#define	SPEED_HEADER		0x00
#define	ORIENTATION_HEADER	0x01
#define	TAG_IN_HEADER		0x02
#define	TAG_OUT_HEADER		0x03
#define	KICK_HEADER			0x04	
#define	RESET_HEADER		0x05
#define KICK_CLEAR_HEADER	0x06

/*---------------------------- Module Functions ---------------------------*/
static void ParseBytes(void);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static PIC_SPI_FSMState_t CurrentState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

static signed char CurrentSpeed = 0x00;
static signed char CurrentOrientation = 0x00;
static unsigned char CurrentAux = 0x00; // auxiliary byte -- low 4 bits are kick,
									    // high 4 bits are brake
unsigned char ReceivedBytes[2];
unsigned char ByteIndex = 0;
unsigned char CurrentStatus = 0x00;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitPIC_SPI_FSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitPIC_SPI_FSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  ANSEL = 0;	// clear ansel to clear analogs
  ANSELH = 0;	// clear anselh to clear analogs
  TRISA = 0;	// set PORTA to outputs

  // already initialized in main -- InitSPI(); // initialize SPI on the PIC

  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitPSPIState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostPIC_SPI_FSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostPIC_SPI_FSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunPIC_SPI_FSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunPIC_SPI_FSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

  switch ( CurrentState )
  {
      case InitPSPIState : 
	  {   
          if ( ThisEvent.EventType == ES_INIT )
          {
              CurrentState = WaitingForSlaveSelectHigh;
           }
	  }
	  break;

      case WaitingForSlaveSelectHigh :
	  {       
          switch ( ThisEvent.EventType )
          {
              case SLAVE_SELECT_HIGH :
              {
				  // if SS line goes high, reset ByteIndex and process ReceivedBytes
					ByteIndex = 0;
                  	ParseBytes();
               }
               break;
          }    
	  }
	  break;
		
/*
      case WaitingForHeaderByte :
	  {       
          switch ( ThisEvent.EventType )
          {
              case BYTE_RECEIVED :
              {
				  NewHeader = ThisEvent.EventParam; // store header byte
				  // PORTA |= BIT2HI; // set A2 (for debugging)
                  CurrentState = WaitingForDataByte;//Decide what the next state will be
               }
               break;
          }    
	  }
	  break;

	  case WaitingForDataByte :
	  {       
          switch ( ThisEvent.EventType )
          {
              case BYTE_RECEIVED :
              {
				  NewData = ThisEvent.EventParam; // store data byte
                  // PORTA &= BIT2LO; // clear A2 (for debugging)
                  ParseBytes();
				  CurrentState = WaitingForSlaveSelect;
               }
               break;
          }
	  }
      break;
*/

  }                                  
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
PIC_SPI_FSMState_t QueryPIC_SPI_FSM ( void )
{
   return(CurrentState);
}

signed char GetCurrentSpeed(void)
{
	return CurrentSpeed;
}

signed char GetCurrentOrientation(void)
{
	return CurrentOrientation;
}

unsigned char GetCurrentStatus(void)
{
	return CurrentStatus;
}

void SetCurrentStatus(unsigned char NewStatus)
{
	CurrentStatus = NewStatus;
}

void ClearCurrentStatus(void)
{
	CurrentStatus = 0x00;
}

unsigned char GetCurrentAux(void)
{
	return CurrentAux;
}

void ClearAux(void)
{
	// clear current aux value
	CurrentAux = 0x00;
}

// process the NewHeader and NewData bytes and respond accordingly
static void ParseBytes(void)
{
	ES_Event ThisEvent;	

	switch( ReceivedBytes[0] )
	{
		case SPEED_HEADER:
		{
			// update current speed
			CurrentSpeed = ReceivedBytes[1];
		}
		break;

		case ORIENTATION_HEADER:
		{
			// update current orientation
			CurrentOrientation = ReceivedBytes[1];
		}
		break;

		case TAG_IN_HEADER:
		{
			// post ES_PAIR_REQUEST event to CoachProtocolFSM module
			ThisEvent.EventType = ES_PAIR_REQUEST;
			ThisEvent.EventParam = ReceivedBytes[1];
			PostCoachProtocolFSM( ThisEvent );
		}
		break;

		case TAG_OUT_HEADER:
		{
			// post TAG_OUT event to CoachProtocolFSM module
			ThisEvent.EventType = ES_TAG_OUT;
			ThisEvent.EventParam = ReceivedBytes[1];
			PostCoachProtocolFSM( ThisEvent );
			//PORTA ^= BIT0HI;
		}
		break;

		case KICK_HEADER:
		{
			//PORTA ^= BIT0HI;
			// update current kick value
			CurrentAux |= 0x0F;
		}
		break;

		case RESET_HEADER:
		{
			//PORTA ^= BIT0HI;
			// post RESET event to CoachProtocolFSM
			ThisEvent.EventType = ES_RESET;
			PostCoachProtocolFSM( ThisEvent );
		}
		break;

		case KICK_CLEAR_HEADER:
		{
			//PORTA ^= BIT0HI;
			// clear current kick value
			CurrentAux &= 0xF0;
		}
		break;

		default:
		{
			// PORTA |= BIT1HI;
		}
		break;
	}
}

/***************************************************************************
 private functions
 ***************************************************************************/

