/****************************************************************************
 Module
   PlayerProtocolFSM.c

 Revision
   1.0.1

 Description
   This does the main processing for the messages we get and
   is the basic state machine for keeping  track or paired status

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PlayerProtocolFSM.h"
#include "PICUartTxFSM.h"
#include "PICUartRxFSM.h"
#include "CommDefines.h"

/*----------------------------- Module Defines ----------------------------*/


#define JERSEY_NUMBER 13

#define ONE_SEC 1000
#define _5HZ 200
#define _10HZ 100

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
static PlayerProtocolState_t WaitLinkProcessPacket( void );
static PlayerProtocolState_t PairedProcessPacket( void );

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static PlayerProtocolState_t CurrentState;
static uint8_t teamColor = 0; // start on the red team (0 = red, 1 = green)
static uint8_t CoachAddr[2]; 


// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitPlayerProtocolFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitPlayerProtocolFSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  MyPriority = Priority;
  // put us into the Initial PseudoState
  CurrentState = InitProtocolState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostPlayerProtocolFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostPlayerProtocolFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTemplateFSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event RunPlayerProtocolFSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

  switch ( CurrentState )
  {
      case InitProtocolState :       // If current state is initial Psedudo State
          if ( ThisEvent.EventType == ES_INIT )// only respond to ES_Init
          {
              // this is where you would put any actions associated with the
              // transition from the initial pseudo-state into the actual
              // initial state

              // now put the machine into the actual initial state
              CurrentState = WaitLink;
           }
           break;

      case WaitLink :       // If current state is WaitLink
          switch ( ThisEvent.EventType )
          {
              case ES_GOOD_PACKET : //If event is event one
              {
                  // Execute action function for state one : event one
                  //PLAYER_DEBUG("Good Packet!\r\n");
                  if( RX_getLock() )
                  {
                     CurrentState = WaitLinkProcessPacket();
                     RX_releaseLock();
                  }
                  
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states
      
      case Paired :       // If current state is WaitLink
          switch ( ThisEvent.EventType )
          {
              case ES_GOOD_PACKET : //If event is event one
              {
                  // Execute action function for state one : event one
                  //PLAYER_DEBUG("Good Packet!\r\n");
                  if( RX_getLock() )
                  {
                     CurrentState = PairedProcessPacket();
                     RX_releaseLock();
                  }
                  
               }
               break;

              // repeat cases as required for relevant events
          }                           // end switch on CurrentEvent
          break;
      // repeat state pattern as required for other states

  }                                   // end switch on Current State
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryPlayerProtocolFSM

 Parameters
     None

 Returns
     PlayerProtocolState_t The current state of the PlayerProtocol state machine

 Description
     returns the current state of the PlayerProtocol state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
PlayerProtocolState_t QueryPlayerProtocolFSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
/****************************************************************************
 Function
     WaitPlayerProcessPacket

 Parameters
     None

 Returns
     None

 Description
     Implment a decision tree to determin the contents of the packet and post appropriate
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
 static PlayerProtocolState_t WaitLinkProcessPacket( void )
 {
   ES_Event TXEvent;
   //if API identifier is TX status
   if(RX_getXBeePacket(XBEE_API) == 0x89)
   {
      ES_Event TXStatusEvent;
      //if TX was a success
      if(RX_getXBeePacket(TXS_STATUS) == 0)
      {
         TXStatusEvent.EventType = ES_TX_STATUS_GOOD;
         PostPICUartTxFSM(TXStatusEvent);
         //PLAYER_DEBUG("TX Status Good\r\n");
         return WaitLink;
      }
      else
      {
         TXStatusEvent.EventType = ES_TX_STATUS_ERROR;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Error\r\n");
         return WaitLink;
      }
   }//end TX Status IF
   
   //is it a broadcast?
   if((RX_getXBeePacket(RX_OPT) & BIT2HI) != 0x04)
   {
    //PLAYER_DEBUG("Not a broadcast\r\n");
      return WaitLink;   
   }
   
   //is it a pair request?
   if(RX_getXBeePacket(RX_DATA) != 0x03)
   {
    //PLAYER_DEBUG("Not a pair request\r\n");
      return WaitLink;
   }
   
   //is it our jersey number?
   if((RX_getXBeePacket(RX_DATA + 1) & 0x0F) != JERSEY_NUMBER)
   {
    //PLAYER_DEBUG("Wrong Jersey Number\r\n");
      return WaitLink;
   }
   
   // Is it on our team?
   if((RX_getXBeePacket(RX_DATA + 1) & BIT4HI)>>4 != teamColor)
   {
      ES_Event TXError;
      TXError.EventType = ES_REQUEST_SEND;
      
      TX_clearXBeePacket();
      TX_appendXBeePacket(0x7E);
      TX_appendXBeePacket(0x00);
      TX_appendXBeePacket(0x07);
      TX_appendXBeePacket(0x01);
      TX_appendXBeePacket(0x01);
      TX_appendXBeePacket(RX_getXBeePacket(RX_ADDR0));
      TX_appendXBeePacket(RX_getXBeePacket(RX_ADDR1));
      TX_appendXBeePacket(0x00);
      TX_appendXBeePacket(0x05);
      TX_appendXBeePacket(0x02);
      TX_appendXBeePacket(TX_getChecksum());
      
      PostPICUartTxFSM(TXError);
    //PLAYER_DEBUG("Wrong team color\r\n");
      return WaitLink;
   }
   // Statup function on way to Paired State
   //store the coach address
   CoachAddr[0] = RX_getXBeePacket(RX_ADDR0);
   CoachAddr[1] = RX_getXBeePacket(RX_ADDR1);  
   
   //send a pair response  
   TXEvent.EventType = ES_REQUEST_SEND;
   
   TX_clearXBeePacket();
   TX_appendXBeePacket(0x7E);
   TX_appendXBeePacket(0x00);
   TX_appendXBeePacket(0x07);
   TX_appendXBeePacket(0x01);
   TX_appendXBeePacket(0x01);
   TX_appendXBeePacket(CoachAddr[0]);
   TX_appendXBeePacket(CoachAddr[1]);
   TX_appendXBeePacket(0x00);
   TX_appendXBeePacket(0x05);
   TX_appendXBeePacket(0x01);
   TX_appendXBeePacket(TX_getChecksum());
   
   PostPICUartTxFSM(TXEvent);
 //PLAYER_DEBUG("Going to paired\r\n");
      
   //start heartbeat timer
   ES_Timer_InitTimer(HEARTBEAT_TIMER, ONE_SEC);
   
   //start status timer
   ES_Timer_InitTimer(STATUS_TIMER, _5HZ);
   
   //activate LiFKIM
   //Post to the SPI module...
   
   //start LiFKIM timer
   ES_Timer_InitTimer(LiFKIM_TIMER, _10HZ);
   
   return Paired;
 }


/****************************************************************************
 Function
     PairedPlayerProcessPacket

 Parameters
     None

 Returns
     None

 Description
     Implement a decision tree to determine the contents of the packet and post appropriate
     states and event
 Notes

 Author
     N. Martelaro, 05/11/14, 21:36
****************************************************************************/
 static PlayerProtocolState_t PairedProcessPacket( void )
 {
   ES_Event TXEvent;
   //if API identifier is TX status
   if(RX_getXBeePacket(XBEE_API) == 0x89)
   {
      ES_Event TXStatusEvent;
      //if TX was a success
      if(RX_getXBeePacket(TXS_STATUS) == 0)
      {
         TXStatusEvent.EventType = ES_TX_STATUS_GOOD;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Good\r\n");
         return Paired;
      }
      else
      {
         TXStatusEvent.EventType = ES_TX_STATUS_ERROR;
         PostPICUartTxFSM(TXStatusEvent);
       //PLAYER_DEBUG("TX Status Error\r\n");
         return Paired;
      }
   }//end TX Status IF
   
   //is it a broadcast?
   if((RX_getXBeePacket(RX_OPT) & BIT2HI) == 0x04)
   {
    //PLAYER_DEBUG("Was a broadcast\r\n");
      //is is a pair request
      if(RX_getXBeePacket(RX_DATA) != 0x03)
      {
       //PLAYER_DEBUG("Not a pair request\r\n");
         //is it a tag detected?
         if(RX_getXBeePacket(RX_DATA) != 0x07)
         {
          //PLAYER_DEBUG("Not a tag detected\r\n");
            return Paired;
         }
         else
         {
          //PLAYER_DEBUG("Relay to LiFKIM\r\n");
            // TODO: relay to LiFKIM via SPI
            return Paired;
         }
      }
      else
      {
       //PLAYER_DEBUG("Pair Request\r\n");
         //is it our jersey number?
         if((RX_getXBeePacket(RX_DATA + 1) & 0x0F) != JERSEY_NUMBER)
         {
          //PLAYER_DEBUG("Wrong Jersey Number\r\n");
            return Paired;
         }
         else
         {
            ES_Event TXError;
            TXError.EventType = ES_REQUEST_SEND;
      
            TX_clearXBeePacket();
            TX_appendXBeePacket(0x7E);
            TX_appendXBeePacket(0x00);
            TX_appendXBeePacket(0x07);
            TX_appendXBeePacket(0x01);
            TX_appendXBeePacket(0x01);
            TX_appendXBeePacket(RX_getXBeePacket(RX_ADDR0));
            TX_appendXBeePacket(RX_getXBeePacket(RX_ADDR1));
            TX_appendXBeePacket(0x00);
            TX_appendXBeePacket(0x05);
            TX_appendXBeePacket(0x04);
            TX_appendXBeePacket(TX_getChecksum());
            
            PostPICUartTxFSM(TXError);
          //PLAYER_DEBUG("Already Paired\r\n");
            return Paired;
         }
      }
   }
   
   //Not a broadcast
   //Is it from our coach?
   if(RX_getXBeePacket(RX_ADDR0) != CoachAddr[0])
   {
    //PLAYER_DEBUG("Wrong Coach MSB\r\n");
      return Paired;
   }
   if(RX_getXBeePacket(RX_ADDR1) != CoachAddr[1])
   {
    //PLAYER_DEBUG("Wrong Coach LSB\r\n");
      return Paired;
   }
   
   //What type of packet is it?
   switch(RX_getXBeePacket(RX_DATA))
   {
      case HDR_RESET:
      {
         //unpair and reset hardware
       //PLAYER_DEBUG("HDR_RESET\r\n");   
      }break;
      
      case HDR_TAG_OUT:
      {
         //relay to LiFKIM
       //PLAYER_DEBUG("HDR_TAG_OUT\r\n");   
      }break;
      
      case HDR_CTRL:
      {
         //update the control outputs
       //PLAYER_DEBUG("HDR_CTRL\r\n");   
      }break;
   }
   
   return Paired;   
}